#! /usr/bin/env python
# -*- Mode: python -*-
#
# Copyright 2012 IFIC (Instituto de Fisica Corpuscular). Distributed under the GPL.
# Author: Victor Ruben Lacort Pellicer <Victor.Lacort@ific.uv.es>
#===================================================================================
#
# lustre-pools.py: Obtiene la lista de pools de cada lustre fsname y sus OSTs 
#
#===================================================================================

import sys, os, re
import time
import socket
import getopt

def usage ():

    print >> sys.stderr, "uso: %s [opciones] fichero" % sys.argv[0]
    print >> sys.stderr, "opciones:"
    print >> sys.stderr, "  -h, --help            muestra este mensaje y sale"
    print >> sys.stderr, "  -v, --verbose         salida detallada"

#esta la modificare para obtener el OSS name
def getLustreOSShost_MGS(fs, ost):

    obdl=[]
    try:
        f=open("/proc/fs/lustre/osc/%s-%s-osc/ost_conn_uuid"%(fs,ost),'r')
    except:
        print >> sys.stderr, "No puedo obtener la informacion del sistema de ficheros ",fs
        sys.exit(2)
        return obdl
    #print "%s %s %s"%(fs,ost,idfs)###
    for l in f:
        host = l.rstrip().replace("@tcp","")
        r = socket.gethostbyaddr(host)
        obdl.append(r[0].split('.')[0])
        #print "%s\t\t%s\t%s"%(ost,r[0],host)
    f.close()
    return obdl


def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hv", ["help", "verbose"])
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err) # will print something like "option -a not recognized"
        usage()
        sys.exit(2)
        

    verbose = False

    for o, a in opts:
        if o in ("-v", "--verbose"):
            verbose = True
        elif o in ("-h", "--help"):
            usage()
            sys.exit()
        else:
            assert False, "unhandled option"

    fsnames = []
    pools   = {}

    osts    = []
    fsosts  = {}

    #1-> cat /proc/fs/lustre/devices -> fsnames[]
    
    f = open("/proc/fs/lustre/devices",'r')
    for l in f:
        tk=l.strip().split()
        if len(tk) == 0:
            continue
	if tk[2] == "lov" :
            fsnames.append(tk[3].split('-')[0])
    f.close()
    
    #2-> lctl pool_list <fsname> -> pools{}
    
    for fs in fsnames:
        f = os.popen ("/usr/sbin/lctl pool_list %s" % fs)
        aux = []
        for l in f:
            tk=l.strip().split()
            if len(tk) == 0:
                continue
            if tk[0] == "Pools" :
                continue
            aux.append(tk[0])
        pools[fs] = aux
        f.close()
        
    #3-> lfs pool_list <fsname>.<poolname> -> osts[]
    for fs in fsnames:
        for poolname in pools[fs]:
            f = os.popen ("/usr/sbin/lctl pool_list %s" % poolname)
            aux = []
            for l in f:
                tk=l.strip().split()
                if tk[0] == "Pool:":
                    continue
                aux.append(tk[0].split('-')[1].split('_')[0])
            osts.append(aux)
            f.close()

    #4-> Lista de OSTs por cada fsname -> fsosts{}
    f = open("/proc/fs/lustre/devices",'r')
    for l in f:
        tk=l.strip().split()
        if len(tk) == 0:
            continue
        if tk[2] == "lov":
            fs = tk[3].split('-')[0]
            fsosts [fs] = []
	if tk[2] == "osc":
            fs = tk[3].split('-')[0]
            fsosts [fs].append(tk[3].split('-')[1])
    f.close()

    #5-> Mostrar Salida
    verb_list  = []
    for fs in fsnames:
        list_pools = "  Pools:      |"
        if pools[fs] != []:
            print "\nFilesystem: \"%s\"" %fs #.split('.')[0]
            for p in pools[fs]:
                name = p.split('.')[1]
                if len(name)>10:
                    name=".."+name[-8:]
                list_pools = list_pools + "%10s |" % name #'\t'
            list_pools += "   NONE  |"
            print list_pools + "\n"

            if fsosts[fs] != []:
                for p in fsosts[fs]:
                    nodo  = getLustreOSShost_MGS(fs, p)
                    linea = nodo[0] + " " + p + " |"
                    none = 1
                    for k in range(len(pools[fs])):
                        if osts[k].count(p):
                            linea += "     X     |"
                            none  = 0
                        else:
                            linea += "           |"
                    if none == 1:
                        linea += "    N    |"
                    else:
                        linea += "         |"
                    print linea #+ '\n'

        else: #para el verbose
            verb_list.append(fs)
            if verbose:
                print "\nFilesystem: \"%s\"" %fs
                for p in pools[fs]:
                    name = p.split('.')[1]
                    if len(name)>10:
                        name=".."+name[-8:]
                    list_pools = list_pools + "%10s |" % name #'\t'
                list_pools += "   NONE  |"
                print list_pools + "\n"

                if fsosts[fs] != []:
                    for p in fsosts[fs]:
                        nodo  = getLustreOSShost_MGS(fs, p)
                        linea = nodo[0] + " " + p + " |"
                        none = 1
                        for k in pools[fs]:
                            if k.count(p):
                                linea += "     X     |"
                                none  = 0
                            else:
                                linea += "           |"
                        if none == 1:
                            linea += "    N    |"
                        else:
                            linea += "         |"
                        print linea #+ '\n'
                else:
                    print "no tiene OSTs activos."

    if not verbose:
        for fs in verb_list:
            print "Filesystem \"%s\" no tiene pools." % fs

#fin
###################################

if __name__ == '__main__':
    main()

